package org.aleabodo.fireworksmc.Commands;

import org.aleabodo.fireworksmc.Direction;
import org.aleabodo.fireworksmc.Fireworks_Mc;
import org.aleabodo.fireworksmc.Message;
import org.aleabodo.fireworksmc.Tempo;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

public class Fireworkshow implements CommandExecutor, TabCompleter {
    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if (!(sender instanceof Player)) {
            sender.sendMessage("This command is reserved for players only!");
            return false;
        }

        Player player = ((Player) sender).getPlayer();

        switch (args[0]) {
            case "setlocation":
                //Set location
                Fireworks_Mc.config.location = player.getLocation();
                Fireworks_Mc.config.save();
                Message.sendInfo(player, "Location successfully set to your position.");
                return true;
            case "setdirection":
                //Compute direction based on player yaw
                if (args.length < 2) {
                    float yaw = player.getLocation().getYaw();
                    if (yaw < 0) {
                        yaw += 360;
                    }
                    yaw = (yaw % 360 + 360) % 360; // true modulo, as javas modulo is weird for negative values
                    Direction direction;
                    if (yaw >= 315 || yaw < 45) {
                        direction = Direction.SOUTH;
                    } else if (yaw < 135) {
                        direction = Direction.WEST;
                    } else if (yaw < 225) {
                        direction = Direction.NORTH;
                    } else if (yaw < 315) {
                        direction = Direction.EAST;
                    } else {
                        direction = Direction.NORTH;
                    }
                    Fireworks_Mc.config.direction = direction;
                    Message.sendInfo(player, "Direction successfully set to " + direction);
                    Fireworks_Mc.config.save();
                    return true;
                }

                //Set direction manually
                switch (args[1]) {
                    case "NORTH":
                        Fireworks_Mc.config.direction = Direction.NORTH;
                        Message.sendInfo(player, "Direction successfully set to " + Direction.NORTH);
                        Fireworks_Mc.config.save();
                        return true;
                    case "EAST":
                        Fireworks_Mc.config.direction = Direction.EAST;
                        Message.sendInfo(player, "Direction successfully set to " + Direction.EAST);
                        Fireworks_Mc.config.save();
                        return true;
                    case "SOUTH":
                        Fireworks_Mc.config.direction = Direction.SOUTH;
                        Message.sendInfo(player, "Direction successfully set to " + Direction.SOUTH);
                        Fireworks_Mc.config.save();
                        return true;
                    case "WEST":
                        Fireworks_Mc.config.direction = Direction.WEST;
                        Message.sendInfo(player, "Direction successfully set to " + Direction.WEST);
                        Fireworks_Mc.config.save();
                        return true;
                    default:
                        Message.sendWarning(player, "Please enter either 'NORTH', 'EAST', 'SOUTH', 'WEST', or leave empty!");
                        return false;
                }
            case "setduration":
                //Set duration
                try {
                    int duration = Integer.parseInt(args[1]);
                    Fireworks_Mc.config.duration = duration;
                    Fireworks_Mc.config.save();
                    Message.sendInfo(player, "Duration successfully set to " + duration);
                    return true;
                } catch (NumberFormatException e) {
                    Message.sendWarning(player, "Please enter an integer for the duration (in seconds)!");
                }
            case "setseed":
                //Set seed
                StringBuilder result = new StringBuilder(args[1]);
                for (int i = 1; i<args.length; i++) {
                    result.append(" ").append(args[i]);
                }
                Fireworks_Mc.config.seed = String.valueOf(result);
                Fireworks_Mc.config.save();
                Message.sendInfo(player, "Seed successfully changed to '" + result + "'");
                return true;
            case "highlights":
                if (args.length < 2) {
                    Message.sendWarning(player, "Invalid command!");
                    return false;
                }
                switch (args[1]) {
                    case "add":
                        if (args.length < 3 || !args[2].matches("^[1-9]\\d*$")) {
                            Message.sendWarning(player, "Please enter a number in game-ticks for the delay!");
                            return false;
                        }

                        Fireworks_Mc.config.highlights.add(Integer.parseInt(args[2]));
                        Fireworks_Mc.config.save();
                        Message.sendInfo(player, "Highlight with a delay of " + args[2] + " game-ticks successfully added.");
                        return true;
                    case "remove":
                        if (args.length < 3 || !args[2].matches("^[1-9]\\d*$")) {
                            Message.sendWarning(player, "Please enter a number in game-ticks for the delay of the highlight you want to remove!");
                            return false;
                        }

                        Fireworks_Mc.config.highlights.remove(Integer.parseInt(args[2]));
                        Fireworks_Mc.config.save();
                        Message.sendInfo(player, "Highlight successfully with delay of " + args[2] + " game-ticks successfully removed.");
                        return true;
                    default:
                        Message.sendWarning(player, "Invalid command!");
                        return false;
                }
            case "tempo":
                if (args.length < 2) {
                    Message.sendWarning(player, "Invalid command!");
                    return false;
                }
                String[] splits; //Used for splitting the input on ","
                switch (args[1]) {
                    case "add":
                        if (args.length < 4) {
                            Message.sendWarning(player, "Please enter a tempo and delay.");
                            return false;
                        }

                        if (!args[2].matches("^[1-9]\\d*$")) {
                            Message.sendWarning(player, "Invalid delay format. Expected an integer.");
                            return false;
                        }

                        Tempo tempo;
                        switch (args[3]) {
                            case "NORMAL":
                                tempo = Tempo.NORMAL;
                                break;
                            case "QUIET":
                                tempo = Tempo.QUIET;
                                break;
                            case "FAST":
                                tempo = Tempo.FAST;
                                break;
                            case "FASTEST":
                                tempo = Tempo.FASTEST;
                                break;
                            case "SLOW":
                                tempo = Tempo.SLOW;
                                break;
                            default:
                                Message.sendWarning(player, "Invalid tempo format! Please enter either 'QUIET', 'SLOW', 'NORMAL', 'FAST', or 'FASTEST'");
                                return false;
                        }

                        Fireworks_Mc.config.tempo.put(Integer.parseInt(args[2]),tempo);
                        Fireworks_Mc.config.save();
                        Message.sendInfo(player, "Tempo '"+ tempo +"' with a delay of " + Integer.parseInt(args[2]) + " game-ticks successfully added.");
                        return true;
                    case "remove":
                        if (args.length < 3) {
                            Message.sendWarning(player, "Please enter a tempo like '20,NORMAL' of the tempo you want to remove. It's also enough to just add the delay number.");
                            return false;
                        }

                        splits = args[2].split(",");
                        if (splits.length < 2 || !splits[0].matches("^[1-9]\\d*$")) {
                            Message.sendWarning(player, "Please enter an integer delay in game-ticks for the tempo you want to remove. ");
                            return false;
                        }

                        Fireworks_Mc.config.tempo.remove(Integer.parseInt(splits[0]));
                        Fireworks_Mc.config.save();
                        Message.sendInfo(player, "Tempo successfully with delay of " + splits[0] + " game-ticks successfully removed.");
                        return true;
                    default:
                        Message.sendWarning(player, "Invalid command!");
                        return false;
                }
            case "start":
                Fireworks_Mc.show.start();
                return true;
            case "generate":
                Message.sendInfo(player, "Generating firework show...");
                Fireworks_Mc.show.generate();
                Message.sendInfo(player, "Firework show successfully generated!");
                return true;
            case "debug":
                Fireworks_Mc.config.print();
                return true;
            case "reload":
                Fireworks_Mc.config.reload();
                return true;
            default:
                Message.sendWarning(player, "Invalid command!");
        }

        return false;
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command,
                                                @NotNull String label, @NotNull String[] args) {
        List<String> completions = new ArrayList<>();

        if (args.length == 1) {
            completions.add("setlocation");
            completions.add("setdirection");
            completions.add("setduration");
            completions.add("setseed");
            completions.add("start");
            completions.add("generate");
            completions.add("debug");
            completions.add("reload");
            completions.add("highlights");
            completions.add("tempo");
        } else if (args.length == 2) {
            if (args[0].equals("setdirection")) {
                completions.add("NORTH");
                completions.add("EAST");
                completions.add("SOUTH");
                completions.add("WEST");
            } else if (args[0].equals("highlights") || args[0].equals("tempo")) {
                completions.add("add");
                completions.add("remove");
            }
        } else if (args.length == 3) {
            if (args[0].equals("tempo") && args[1].equals("remove")) {
                for (Integer key : Fireworks_Mc.config.tempo.keySet()) {
                    completions.add(key.toString() + "," + Fireworks_Mc.config.tempo.get(key).toString());
                }
            }
            if (args[0].equals("highlights") && args[1].equals("remove")) {
                for (Integer h : Fireworks_Mc.config.highlights) {
                    completions.add(h.toString());
                }
            }
        } else if (args.length == 4) {
            if (args[0].equals("tempo") && args[1].equals("add")) {
                completions.add("QUIET");
                completions.add("SLOW");
                completions.add("NORMAL");
                completions.add("FAST");
                completions.add("FASTEST");
            }
        }

        return completions;
    }
}
