package org.aleabodo.fireworksmc;

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Location;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;

import java.util.*;
import java.util.logging.Level;

public class Config {
    public int bpm;
    public String seed;
    public int duration;
    public Location location;
    public Direction direction;
    public SortedSet<Integer> highlights;
    public SortedMap<Integer,Tempo> tempo;

    public Config() {
        reload();
    }

    private void load() {
        Fireworks_Mc.getPlugin().reloadConfig();
        FileConfiguration config = Fireworks_Mc.getPlugin().getConfig();
        this.bpm = config.getInt("bpm", 80);
        this.duration = config.getInt("duration", 60);
        this.seed = config.getString("seed", "Survival-Pi");
        this.highlights = new TreeSet<>(config.getIntegerList("highlights"));

        // Parse and set location
        this.location = new Location(
                Fireworks_Mc.getWorld(config.getString("location.world","world")),
                config.getInt("location.x",0),
                config.getInt("location.y",80),
                config.getInt("location.z",0)
        );

        // Parse and set direction
        String direction = config.getString("direction", "WEST");
        if(direction != null) {
            switch (direction) {
                case "NORTH":
                    this.direction = Direction.NORTH;
                case "EAST":
                    this.direction = Direction.EAST;
                case "SOUTH":
                    this.direction = Direction.SOUTH;
                default:
                    this.direction = Direction.WEST;
            }
        }

        // Parse and set tempo
        SortedMap<Integer,Tempo> tempo = new TreeMap<>();
        List<String> tempoArray = config.getStringList("tempo");
        tempo.put(0, Tempo.NORMAL); //Have at least one tempo configuration in the map
        for (String tempoString : tempoArray) {
            String[] args = tempoString.split(",");
            if (args.length == 2) {
                try {
                    Tempo t;
                    switch (args[1]) {
                        case "QUIET":
                            t = Tempo.QUIET;
                        case "SLOW":
                            t = Tempo.SLOW;
                        case "FAST":
                            t = Tempo.FAST;
                        case "FASTEST":
                            t = Tempo.FASTEST;
                        default:
                            t = Tempo.NORMAL;
                    }
                    tempo.put(Integer.parseInt(args[0]), t);
                } catch (NumberFormatException e) {
                    Fireworks_Mc.getPlugin().getLogger().log(Level.SEVERE, "Config in section 'tempo' is expecting an integer!");
                    return;
                }
            }
        }
        this.tempo = tempo;
    }

    public void save() {
        FileConfiguration config = Fireworks_Mc.getPlugin().getConfig();

        config.set("bpm", this.bpm);
        config.set("seed", this.seed);
        config.set("duration", this.duration);
        config.set("direction", this.direction.toString());
        config.set("highlights", new ArrayList<>(this.highlights));

        //Convert tempo map to ArrayList and set tempo in config to this list
        ArrayList<String> tempoStringList = new ArrayList<>();
        for (Integer key : this.tempo.keySet()) {
            Tempo value = this.tempo.get(key);
            tempoStringList.add(key + "," + value.toString());
        }
        config.set("tempo", tempoStringList);

        ConfigurationSection locationSection = config.createSection("location");
        locationSection.set("world", this.location.getWorld().getName());
        locationSection.set("x", this.location.getX());
        locationSection.set("y", this.location.getY());
        locationSection.set("z", this.location.getZ());

        Fireworks_Mc.getPlugin().saveConfig();
    }

    public void print() {
        //Compute string of all highlights. Use this string to append to the text component
        StringBuilder highlight = new StringBuilder();
        for (Integer h : this.highlights) {
            highlight.append("\n")
                    .append("  - ")
                    .append(".".repeat(Message.getNumberOfDotsToAlign("  - " + h)))
                    .append(h);
        }

        //Compute string of all tempos
        StringBuilder tempos = new StringBuilder();
        for (Integer key : this.tempo.keySet()) {
            Tempo value = this.tempo.get(key);
            tempos.append("\n")
                    .append("  - ")
                    .append(".".repeat(Message.getNumberOfDotsToAlign("  - " + key + "," + value.toString())))
                    .append(key).append(",").append(value.toString());
        }

        Message.sendDebug(
                Component.text("Config:\n").color(NamedTextColor.GOLD),
                Component.text(
                        "direction: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("direction: " + this.direction.toString()))
                                + this.direction.toString()
                                + "\n"

                                + "bpm: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("bpm: " + this.bpm))
                                + this.bpm
                                + "\n"

                                + "duration: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("duration: " + this.duration))
                                + this.duration
                                + "\n"

                                + "seed: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("seed: " + this.seed))
                                + this.seed
                                + "\n"

                                + "location:"
                                + "\n"
                                + "  - world: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("  - world: " + this.location.getWorld().getName()))
                                + this.location.getWorld().getName()
                                + "\n"
                                + "  - x: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("  - x: " + this.location.getX()))
                                + this.location.getX()
                                + "\n"
                                + "  - y: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("  - y: " + this.location.getY()))
                                + this.location.getY()
                                + "\n"
                                + "  - z: "
                                + ".".repeat(Message.getNumberOfDotsToAlign("  - z: " + this.location.getZ()))
                                + this.location.getZ()
                                + "\n"

                                + "highlights:"
                                + highlight
                                + "\n"

                                + "tempo:"
                                + tempos
                )
        );
    }

    public void reload() {
        load();
    }
}
